var ListPage = 
{
    id: "contentslp",
    url: "/content/",
    footerImg: "Images/ListPage/footer.png",
    enableNumberInput: true,
    visibleItems: 4,
    scrollableHeight: 392,

    data: null,
    curColumn: 0,
    curContent: 0,
    curVideo: 0,
    contentOffset: 0,
    videoOffset: 0,
    descriptionOffset: 0,
    errorMessage: "{itemNameGC} {number} в {namePC} &laquo;{title}&raquo; "
                  + "не существует. Ниже расположены все {itemNamePlural} "
                  + "({first}&mdash;{last}) по порядку.",

    COLS: {
        first: 0,
        content: 0,
        video: 1,
        description: 2,
        last: 2
    }
};

ListPage.init = function()
{
    ListPage.curColumn = 0;
    ListPage.curContent = 0;
    ListPage.curVideo = 0;
    ListPage.contentOffset = 0;
    ListPage.videoOffset = 0;
    ListPage.descriptionOffset = 0;
    return true;
};

ListPage.request = function(contentType) 
{
    Main.request(ListPage.url + contentType, ListPage.activate);
};

ListPage.setVisible = function(visible)
{
    var display = "none";

    if (visible) {
        display = "block";
        Main.setFooterImg(ListPage.footerImg);
        Main.setBreadcrumbs(ListPage.data.title);
    }
    else {
        ListPage.hideNumberInputMessage();
    }

    document.getElementById(ListPage.id).style.display = display;
};

ListPage.activate = function(data)
{
    ListPage.init();
    ListPage.data = data;

    ListPage.selectContent(1);
    Main.setCurrentPage(ListPage);
};

ListPage.onLeft = function()
{
    ListPage.hideNumberInputMessage();
    ListPage.selectColumn(ListPage.curColumn - 1);
};

ListPage.onRight = function()
{
    ListPage.hideNumberInputMessage();

    if (ListPage.curColumn == ListPage.COLS.content
        && ListPage.curColumn + 1 == ListPage.COLS.video)
    {
        // apply usability approved custom rules of curVideo selection
        var video = ListPage.curContent;
        if (video == 0 && ListPage.videoOffset > 0) {
            video++;
        }
        var videosLength = ListPage.getSelectedContent().videos.length;
        if (video == ListPage.visibleItems - 1
            && videosLength - ListPage.videoOffset - ListPage.visibleItems > 0)
        {
            video--;
        }
        if (video > videosLength - 1) {
            video = videosLength;
        }
        ListPage.curVideo = video;
    }

    ListPage.selectColumn(ListPage.curColumn + 1);
};

ListPage.onUp = function()
{
    ListPage.hideNumberInputMessage();

    if (ListPage.curColumn == ListPage.COLS.content) {
        ListPage.selectContent(ListPage.curContent - 1);
    }
    else if (ListPage.curColumn == ListPage.COLS.video) {
        ListPage.selectVideo(ListPage.curVideo - 1);
    }
    else if (ListPage.curColumn == ListPage.COLS.description) {
        ListPage.scrollDescription(ListPage.descriptionOffset - 30);
    }
};

ListPage.onDown = function()
{
    ListPage.hideNumberInputMessage();

    if (ListPage.curColumn == ListPage.COLS.content) {
        ListPage.selectContent(ListPage.curContent + 1);
    }
    else if (ListPage.curColumn == ListPage.COLS.video) {
        ListPage.selectVideo(ListPage.curVideo + 1);
    }
    else if (ListPage.curColumn == ListPage.COLS.description) {
        ListPage.scrollDescription(ListPage.descriptionOffset + 30);
    }
};

ListPage.onEnter = function()
{
    ListPage.hideNumberInputMessage();

    var content = ListPage.getSelectedContent()
    var video = ListPage.getSelectedVideo();

    if (video) {
        PlayerPage.request(content.contentType, content.contentId,
                           video.videoId);
    }
    else {
        DetailsPage.request(content.contentType, content.contentId);
    }
};

ListPage.onNumberInput = function(num)
{
    ListPage.hideNumberInputMessage();

    var content = ListPage.getSelectedContent();
    var found = false;

    for (var i = 0; i < content.videos.length; i++) {
        if (num == content.videos[i].number) {
            var relativePos = i - ListPage.videoOffset;
            ListPage.selectVideo(relativePos);
            ListPage.selectColumn(ListPage.COLS.video);
            found = true;
            break;
        }
    }

    if (!found) {
        var error = ListPage.errorMessage;
        error = error.replace("{title}", content.title);
        error = error.replace("{namePC}", content.contentNamePC);
        error = error.replace("{itemNameGC}",
                              Utils.capitalize(content.itemNameGC));
        error = error.replace("{itemNamePlural}", content.itemNamePlural);
        error = error.replace("{number}", num);
        error = error.replace("{first}", content.videos[0].number);
        error = error.replace("{last}",
                              content.videos[content.videos.length - 1].number);

        ListPage.videoOffset = -1;
        ListPage.curVideo = 1;
        ListPage.drawVideos();

        var element = Utils.getById("lpcell" + ListPage.COLS.video + 0);
        Utils.addClass(element, "error");
        Utils.putHtml(Utils.getByTag(element, "p"), error);
        element.style.visibility = "visible";
    }
};

ListPage.hideNumberInputMessage = function()
{
    if (ListPage.videoOffset < 0) {
        var element = Utils.getById("lpcell" + ListPage.COLS.video + 0);
        Utils.removeClass(element, "error");
        ListPage.videoOffset = 0;
        ListPage.curVideo = 0;
        ListPage.drawVideos();
    }
}

/* Returns selected by user ContentItem. */
ListPage.getSelectedContent = function()
{
    var index = ListPage.curContent + ListPage.contentOffset;
    return ListPage.data.contentItems[index];
};

/* Returns selected by user VideoItem or null. */
ListPage.getSelectedVideo = function()
{
    var video = null;

    if (ListPage.curColumn == ListPage.COLS.video) {
        var index = ListPage.curVideo + ListPage.videoOffset;
        video = ListPage.getSelectedContent().videos[index];
    }

    return video;
};

ListPage.selectColumn = function(col)
{
    if (col >= ListPage.COLS.first && col <= ListPage.COLS.last) {
        ListPage.curColumn = col;
        ListPage.drawContent();
        ListPage.drawVideos();
        ListPage.drawDescription();
    }
};

/* Selects content by relative to current offset position. */
ListPage.selectContent = function(contentPos)
{
    scroller = ListPage.getScroller(ListPage.data.contentItems.length,
                                    ListPage.contentOffset, contentPos);
    ListPage.contentOffset = scroller.offset;
    ListPage.curContent = scroller.position;

    ListPage.drawContent();

    ListPage.descriptionOffset = 0;
    ListPage.drawDescription();

    ListPage.videoOffset = 0;
    ListPage.selectVideo(0);
};

/* Selects video by relative to current offset position. */
ListPage.selectVideo = function(videoPos)
{
    var content = ListPage.getSelectedContent();
    scroller = ListPage.getScroller(content.videos.length,
                                    ListPage.videoOffset, videoPos);
    ListPage.videoOffset = scroller.offset;
    ListPage.curVideo = scroller.position;

    ListPage.drawVideos();
};

/* Computes how scrolling must be done by offset and relative to that
 * offset position. Returns object with the `offset` and `position`
 * attributes. */
ListPage.getScroller = function(length, offset, position)
{
    // get absolute position within [0, length) interval
    var absPos = position + offset;
    absPos = Math.min(Math.max(absPos, 0), length-1);

    // boundaries of the preferable interval to show item in
    var innerStart = offset + 1;
    var innerEnd = offset + ListPage.visibleItems - 1;

    if (absPos < innerStart) {
        innerStart = absPos;
    }
    else if (absPos >= innerEnd) {
        innerEnd = absPos + 1;
        innerStart = innerEnd - ListPage.visibleItems + 2;
    }
    else {
        // absPos is withing preferable interval so we do nothing here
    }

    var newOffset = Math.max(innerStart - 1, 0);
    if (length < ListPage.visibleItems) {
        newOffset = 0;
    }
    else {
        newOffset = Math.min(newOffset, length - ListPage.visibleItems);
    }

    return {offset: newOffset, position: absPos - newOffset};
};

ListPage.drawContent = function()
{
    var columnNode = Utils.getById("lpcol" + ListPage.COLS.content);
    var colSelected = (ListPage.curColumn == ListPage.COLS.content);
    if (colSelected) {
        Utils.removeClass(columnNode, "dimmed");
    }
    else {
        Utils.addClass(columnNode, "dimmed");
    }
    
    for(var i = 0; i < ListPage.visibleItems; i++)
    {
        var content = ListPage.data.contentItems[i + ListPage.contentOffset];
        var element = Utils.getById("lpcell" + ListPage.COLS.content + i);

        if (i == ListPage.curContent) {
            Utils.addClass(element, colSelected ? "selected" : "highlighted");
            Utils.removeClass(element, colSelected ? "highlighted" : "selected");
        }
        else {
            Utils.removeClass(element, "selected");
            Utils.removeClass(element, "highlighted");
        }

        if (content) {
            element.style.visibility = "visible";
            Utils.putHtml(Utils.getByTag(element, "h2"), content.title);
            Utils.putHtml(Utils.getByTag(element, "p"), content.description);
            Utils.setImage(Utils.getByTag(element, "img"), content.image);
        }
        else {
            element.style.visibility = "hidden";
        }
    }
};

ListPage.drawVideos = function()
{
    var columnSelected = (ListPage.curColumn == ListPage.COLS.video);
    var content = ListPage.getSelectedContent();

    for(var i = 0; i < ListPage.visibleItems; i++)
    {
        var video = content.videos[i + ListPage.videoOffset];
        var element = Utils.getById("lpcell" + ListPage.COLS.video + i);

        if (i == ListPage.curVideo && columnSelected) {
            Utils.addClass(element, "selected");
        }
        else {
            Utils.removeClass(element, "selected");
        }

        if (video) {
            var duration = "Время: " + Utils.parseTime(video.duration, true)

            element.style.visibility = "visible";
            Utils.putHtml(Utils.getByTag(element, "h2"), video.title);
            Utils.putHtml(Utils.getByTag(element, "h3"), duration);
            Utils.putHtml(Utils.getByTag(element, "span"), video.number);
            Utils.setImage(Utils.getByTag(element, "img"), video.image);
        }
        else {
            element.style.visibility = "hidden";
        }
    }
};

ListPage.drawDescription = function()
{
    var scrollable = Utils.getById("lp_scroller");
    var margin = String(-ListPage.descriptionOffset) + "px";
    if (margin != scrollable.style.marginTop) {
        scrollable.style.marginTop = margin;
    }

    var content = ListPage.getSelectedContent();
    var element = Utils.getById("lpcol" + ListPage.COLS.description);
    Utils.putHtml(Utils.getById("lpdesc_container"), content.fullDescription);
    Utils.setImage(Utils.getByTag(element, "img"), content.image);

    if (ListPage.curColumn == ListPage.COLS.description) {
        Utils.addClass(element, "selected");
    }
    else {
        Utils.removeClass(element, "selected");
    }
};

ListPage.scrollDescription = function(offset)
{
    var descHeight = Utils.getById("lp_scroller").clientHeight;

    if (offset < 0) {
        offset = 0;
    }

    if (offset < ListPage.descriptionOffset){
        ListPage.descriptionOffset = offset;
        ListPage.drawDescription();
    }
    else if (descHeight - ListPage.descriptionOffset >= ListPage.scrollableHeight
             && offset > ListPage.descriptionOffset)
    {
        ListPage.descriptionOffset = offset;
        ListPage.drawDescription();
    }
};

ListPage.onPanVolUp = function()
{
    ListPage.onRight();
};

ListPage.onPanVolDown = function()
{
    ListPage.onLeft();
};

ListPage.onPanChUp = function()
{
    ListPage.onUp();
};

ListPage.onPanChDown = function()
{
    ListPage.onDown();
};

ListPage.notifyStat = function(skipped)
{
    var statObj = {
        event_type: "page_view",
        event_info: ListPage.data.title,
        //content_type: ListPage.contentType,
        //content_id: ListPage.contentId,
    };
    if (skipped) {
        statObj.event_info = "(ret)";
    }
    Stat.notify(statObj);

};
