var widgetAPI = new Common.API.Widget();
var pluginAPI = new Common.API.Plugin();
var tvKey = new Common.API.TVKeyValue();

var Main =
{
    config: {
        debug: false,
        appName: "samsung",
        version: "1.001",
        apiUrl: "http://stream.ru/api/widget",
        //apiUrl: "http://tvobla.tst.drupal.rwsit.net/api/widget",
        modules: ['Utils', 'Audio', 'Server', 'Stat', 'FirstPage',
                  'DetailsPage', 'ListPage', 'PlayerPage', 'AlertPage'],
        initError: "Ошибка инициализации модуля: {name}"
    },

    /* Stores "number input mode" data. To enable that mode page must
     * define `enableNumberInput` variable and `onNumberInput`
     * function. */
    numbers: {
        input: [],
        maxInputLen: 3,
        placeholder: "-",
        acceptDelay: 2000,
        acceptTimer: null,
        consumeEntersDelay: 1000,
        consumeEntersTimer: null,
    },

    currentPage: null,
    pageHistory: [],
    isWaitingForRequest: false, 
};

Main.onLoad = function()
{
    var inited = Main.initModules();
    Main.enableKeys();

    if (inited) {
        FirstPage.request();
    }
    widgetAPI.sendReadyEvent();

    Main.afterReadyInit();
};

Main.afterReadyInit = function()
{
    Stat.initDeviceId();
}

Main.onUnload = function()
{
    // call deinit of all moduels
    var statObj = {
        event_type: "exit",
    };
    Stat.notify(statObj);

    for (var i = 0; i < Main.config.modules.length; i++)
    {
        var moduleName = Main.config.modules[i];
        if (typeof window[moduleName] != "undefined"
            && typeof window[moduleName]['deinit'] == "function")
        {
            window[moduleName]['deinit']();
        }
    }
};

/* Must be called to set new current page. skipHistory is false by
 * default. */
Main.setCurrentPage = function(page, skipHistory)
{
    if(typeof skipHistory == "undefined") {
        var skipHistory = false;
    }

    if (Main.currentPage) {
        Main.currentPage.setVisible(false);

        if(!skipHistory) {
            Main.pageHistory.push(Main.currentPage);
        }
    }

    Main.setFooterImg(null);
    Main.setBreadcrumbs(null);
    Main.resetNumberInputMode();
    Main.currentPage = page;
    page.setVisible(true);
    page.notifyStat(skipHistory);
};

/* Changes footer background image to imageUrl. If imageUrl is falsey
 * it will be cleared. */
Main.setFooterImg = function(imageUrl)
{
    if (imageUrl) {
        imageUrl = "url('" +imageUrl +"')";
    }
    else {
        imageUrl = "";
    }
    document.getElementById("footer").style.backgroundImage = imageUrl;
};

/* Sets breadcrumbs text. If text is falsy, breadcrumbs will be
 * cleared. */
Main.setBreadcrumbs = function(text)
{
    if (!text) {
        text = "";
    }
    Utils.putHtml(Utils.getById("header-subtitle"), text);
};

Main.enableKeys = function()
{
    document.getElementById("anchor").focus();
};

/* User input handler. Calls appropriate Main's handler if it exists
 * otherwise it will try to call handler from Main.currentPage. */
Main.keyDown = function()
{
    if (Main.isWaitingForRequest) {
        Main.debug("Discard user input while waiting for Server.request.");
        return;
    }

    var keyHandlers = {};
    keyHandlers[tvKey.KEY_ENTER] = "onEnter";
    keyHandlers[tvKey.KEY_PANEL_ENTER] = "onEnter";
    keyHandlers[tvKey.KEY_UP] = "onUp";
    keyHandlers[tvKey.KEY_DOWN] = "onDown";
    keyHandlers[tvKey.KEY_LEFT] = "onLeft";
    keyHandlers[tvKey.KEY_RIGHT] = "onRight";
    keyHandlers[tvKey.KEY_RETURN] = "onReturn";
    keyHandlers[tvKey.KEY_PANEL_RETURN] = "onReturn";
    keyHandlers[tvKey.KEY_MENU] = "onMenu";
    keyHandlers[tvKey.KEY_PANEL_MENU] = "onMenu";
    keyHandlers[tvKey.KEY_INFO] = "onInfo";
    keyHandlers[tvKey.KEY_INFOLINK] = "onInfo";
    keyHandlers[tvKey.KEY_FF] = "onFF";
    keyHandlers[tvKey.KEY_RW] = "onRW";
    keyHandlers[tvKey.KEY_PAUSE] = "onPause";
    keyHandlers[tvKey.KEY_PLAY] = "onPlay";
    keyHandlers[tvKey.KEY_STOP] = "onStop";
    keyHandlers[tvKey.KEY_MUTE] = "onMute";
    keyHandlers[tvKey.KEY_PANEL_VOL_UP] = "onPanVolUp";
    keyHandlers[tvKey.KEY_VOL_UP] = "onVolUp";
    keyHandlers[tvKey.KEY_PANEL_VOL_DOWN] = "onPanVolDown";
    keyHandlers[tvKey.KEY_VOL_DOWN] = "onVolDown";
    keyHandlers[tvKey.KEY_CH_UP] = "onChUp";
    keyHandlers[tvKey.KEY_PANEL_CH_UP] = "onPanChUp";
    keyHandlers[tvKey.KEY_CH_DOWN] = "onChDown";
    keyHandlers[tvKey.KEY_PANEL_CH_DOWN] = "onPanChDown";
    keyHandlers[tvKey.KEY_PANEL_SOURCE] = "onReturn";
    keyHandlers[tvKey.KEY_SOURCE] = "onReturn";
    keyHandlers[tvKey.KEY_ASPECT] = "onAspect";
    keyHandlers[tvKey.KEY_GREEN] = "onGreen";

    var numberKeys = {};
    for(var n = 0; n <= 9; n++) {
        numberKeys[tvKey["KEY_" + n]] = n;
    }

    var keyCode = event.keyCode;
    if (keyHandlers[keyCode]) {
        var handlerName = keyHandlers[keyCode];
        Main.debug("Pressed button with handler: " + handlerName);

        if (typeof Main[handlerName] == "function")
        {
            Main[handlerName]();
        }
        else if (Main.currentPage
                 && typeof Main.currentPage[handlerName] == "function")
        {
            Main.currentPage[handlerName]();
        }
    }
    else if (typeof numberKeys[keyCode] == "number") {
        Main.onNumberKey(numberKeys[keyCode]);
    }
    else {
        Main.debug("Unhandled button pressed, keyCode = " + keyCode);
    }
};

Main.debug = function(message)
{
    if (Main.config.debug) {
        alert(message);
    }
};

Main.showError = function(message)
{
    Main.debug("ERROR: " + message);
    AlertPage.show(message);
};

/* Calls init for all objects in Main.config.modules. */
Main.initModules = function()
{
    var result = true;

    for (var i = 0; i < Main.config.modules.length; i++)
    {
        var moduleName = Main.config.modules[i];
        if (typeof window[moduleName] == "undefined") {
            Main.debug("Module is undefined: " + moduleName);
        }
        else if (typeof window[moduleName]['init'] == "function") {
            var success = window[moduleName]['init']();
            if (!success) {
                var msg = Main.config.initError.replace("{name}", moduleName);
                Main.showError(msg);
                result = false;
                break;
            }
        }
    } // for

    return result;
};

/* Returns user to the previous page. */
Main.onReturn = function()
{
    if (Main.currentPage && Main.pageHistory.length) {
        widgetAPI.blockNavigation(event);
        Main.setCurrentPage(Main.pageHistory.pop(), true);
    }
    else {
        Main.debug("Exit by returning from FirstPage.");
    }
};

/* Returns user to the FirstPage. */
Main.onMenu = function()
{
    if (Main.currentPage && Main.currentPage !== FirstPage) {
        Main.pageHistory = [];
        Main.setCurrentPage(FirstPage, true);
    }
};

/* Handles pressed numbers for "nubmer input mode" */
Main.onNumberKey = function(num)
{
    if (Main.currentPage && Main.currentPage.enableNumberInput)
    {
        Main.numbers.input.push(num);
        Main.drawNumberInput();
        if (Main.numbers.input.length == Main.numbers.maxInputLen) {
            Main.setConsumeEntersTimer(true);
            Main.acceptNumber();
        }
        else {
            Main.setAcceptNumberTimer(true);
            Main.setConsumeEntersTimer(false);
        }
    }
};

/* Cancels number input mode and resets all its timers, etc. */
Main.resetNumberInputMode = function()
{
    Main.numbers.input = [];
    Main.drawNumberInput();
    Main.setAcceptNumberTimer(false);
};

/* Ends entering phase of input number mode and sends number to
 * currentPage. */
Main.acceptNumber = function()
{
    if (Main.currentPage) {
        var humaneNum = "";
        for (var i = 0; i < Main.numbers.input.length; i++) {
            humaneNum += String(Main.numbers.input[i]);
        }
        Main.currentPage.onNumberInput(parseInt(humaneNum, 10));
    }

    Main.setAcceptNumberTimer(false);
    Main.numbers.input = [];
    Main.drawNumberInput();
};

Main.setAcceptNumberTimer = function(enabled)
{
    clearTimeout(Main.numbers.acceptTimer);
    Main.numbers.acceptTimer = null;

    if (enabled) {
        var onTimer = function ()
        {
            Main.setConsumeEntersTimer(true);
            Main.acceptNumber();
        }
        Main.numbers.acceptTimer = setTimeout(onTimer,
                                              Main.numbers.acceptDelay);
    }
};

Main.setConsumeEntersTimer = function(enabled)
{
    clearTimeout(Main.numbers.consumeEntersTimer);
    Main.numbers.consumeEntersTimer = null;

    if (enabled) {
        var onTimer = function()
        {
            Main.setConsumeEntersTimer(false);
        }
        Main.numbers.consumeEntersTimer = setTimeout(onTimer,
                                                     Main.numbers.consumeEntersDelay);
    }
};

Main.drawNumberInput = function()
{
    var text = "";
    var display = "none";

    if (Main.numbers.input.length)
    {
        display = "block";
        for (var i = 0; i < Main.numbers.input.length; i++) {
            text += String(Main.numbers.input[i]);
        }
        while (text.length < Main.numbers.maxInputLen) {
            text += Main.numbers.placeholder;
        }
    }

    Utils.putHtml(Utils.getById("header-number-input"), text);
    Utils.getById("header-number-input").style.display = display;
};

/* Handles Enter-key event for "number input mode". It it's not
 * active it will propagate event to currentPage. */
Main.onEnter = function()
{
    if (Main.numbers.input.length) {
        Main.acceptNumber();
    }
    else if (Main.numbers.consumeEntersTimer) {
        Main.setConsumeEntersTimer(false);
    }
    else if (Main.currentPage && typeof Main.currentPage.onEnter == "function") {
        Main.currentPage.onEnter();
    }
};

/* Wrapper for Server.request to handle errors and loading
 * timeouts. Calls callback only if proper JSON data received from
 * server. */
Main.request = function(path, callback)
{
    var onRequestEnd = function(data)
    {
        Main.isWaitingForRequest = false;
        if(data) {
            callback(data);
        }
    };

    Main.isWaitingForRequest = true;
    Server.request(path, onRequestEnd);
};
