var PlayerPage =
{
    states: {
        STOPPED : 0,
        PLAYING : 1,
        PAUSED : 2,  
        FORWARD : 3,
        REWIND : 4,
        BUFFERING : 5
    },
    aspects: {
        ASPECT_16_9: 0,
        ASPECT_4_3: 1,
    },
    id: "contentspp",
    url: "/content/",

    audioPlugin: null,
    bufferingSpin: 250,
    contentId: "",
    contentType: "",
    current_aspect: 0,
    fullscreen: 0,
    mute: false,
    no_volume: false,
    originalSource: null,
    plugin: null,
    prev_state: null,
    response: null,
    state: -1,
    token: "",
    totTime: 0,
    videoId: "",
    watchIntervalId: null,
    watchTimeout: 60000 * 5,
};
        
PlayerPage.init = function()
{
    var success = true;

    PlayerPage.state = PlayerPage.states.STOPPED;
            
    PlayerPage.plugin = document.getElementById("pluginPlayer");
    PlayerPage.tvmw = document.getElementById("pluginTVMW");
    PlayerPage.audioPlugin = document.getElementById("pluginAudio");
    PlayerPage.mute = PlayerPage.audioPlugin.GetSystemMute() && PlayerPage.audioPlugin.GetUserMute();

    PlayerPage.current_aspect = PlayerPage.aspects.ASPECT_16_9;
    PlayerPage.setFullscreen();
    PlayerPage.updateVolume();
    PlayerPage.resetStatus();

    var statusbar = Utils.getById("pp_status_bar");
    Utils.removeClass(statusbar, "pp_full_bar");
    Utils.removeClass(statusbar, "pp_volume_bar");
    Utils.addClass(statusbar, "pp_no_bar");
    PlayerPage.hidePrevNext();
            
    PlayerPage.plugin.OnCurrentPlayTime = 'PlayerPage.setCurrentTime';

    PlayerPage.plugin.OnStreamNotFound = 'PlayerPage.onStreamNotFound';
    PlayerPage.plugin.OnNetworkDisconnected = 'PlayerPage.onNetworkDisconnected';
    PlayerPage.plugin.OnRenderError = 'PlayerPage.onRenderError';
    PlayerPage.plugin.OnConnectionFailed = 'PlayerPage.onConnectionFailed';

    PlayerPage.plugin.OnStreamInfoReady = 'PlayerPage.setTotalTime';
    PlayerPage.plugin.OnBufferingProgress = 'PlayerPage.onBufferingProgress';
    PlayerPage.plugin.OnRenderingComplete = 'PlayerPage.onRenderingComplete';
    PlayerPage.plugin.OnBufferingStart = 'PlayerPage.onBufferingStart';
    PlayerPage.plugin.OnBufferingComplete = 'PlayerPage.onBufferingComplete';           

    if (PlayerPage.audioPlugin.GetVolume() === -1) {
        // we do not have controll over volume
        PlayerPage.no_volume = true;
        var pp = Utils.getById("contentspp");
        Utils.addClass(pp, "pp_bd");
        Utils.removeClass(pp, "pp_tv");
    } else {
        PlayerPage.no_volume = false;
        Utils.addClass(pp, "pp_tv");
        Utils.removeClass(pp, "pp_bd");
    }
                                            
    return true;
};

PlayerPage.resetStatus = function()
{
    var totTime = Utils.parseTime(0);

    Utils.putHtml(Utils.getById("pp_info_tot_time"), totTime);
    Utils.putHtml(Utils.getById("pp_status_tot_time"), totTime);
    PlayerPage.setCurrentTime(0);

    var play = Utils.getById("pp_info_state").getElementsByTagName("img")[0];
    play.src = "Images/PlayerPage/play_small.png";
};

PlayerPage.setTotalTime = function()
{
    PlayerPage.totTime = PlayerPage.plugin.GetDuration();
    var totTime = Utils.parseTime(PlayerPage.totTime);

    Utils.putHtml(Utils.getById("pp_info_tot_time"), totTime);
    Utils.putHtml(Utils.getById("pp_status_tot_time"), totTime);
};

PlayerPage.onBufferingProgress = function(percent)
{
    // Seems like TV can fire BufferingProgress without sending
    // onBuffering start, so let's keep it safe
    if (Player.state !== PlayerPage.states.BUFFERING) {
        PlayerPage.onBufferingStart();
        Main.debug("Buffering progress (" + percent + ")");
    }
};

PlayerPage.onBufferingStart = function()
{
    PlayerPage.state = PlayerPage.states.BUFFERING;
    l = Utils.getById("pp_loading");
    l.style.display = "block";
    if (PlayerPage.bufferingTimer)
        clearInterval(PlayerPage.bufferingTimer);
    PlayerPage.bufferingTimer = setInterval(
            function() {
                img = Utils.getById("pp_loading_progress");
                var found = img.src.match(new RegExp('Images/PlayerPage/loader(\\d+).png'));
                var i = found[1]=="10" ? 1 : parseInt(found[1])+1;
                img.src = "Images/PlayerPage/loader" + i + ".png";
                if (PlayerPage.state !== PlayerPage.states.BUFFERING) {
                    Main.debug("will shoot bufferingComplete");
                    PlayerPage.onBufferingComplete();
                }
            } , 
            PlayerPage.bufferingSpin);
    Main.debug("Buffering started!");
};

PlayerPage.onBufferingComplete = function()
{
    if (PlayerPage.state == PlayerPage.states.BUFFERING) {
        PlayerPage.state = PlayerPage.states.PLAYING;
    } else if (PlayerPage.state == PlayerPage.state.PAUSED) {
        PlayerPage.plugin.Pause();
    }
    l = Utils.getById("pp_loading");
    l.style.display = "none";
    clearInterval(PlayerPage.bufferingTimer);
    Main.debug("Buffering complete!");
};

PlayerPage.onStreamNotFound = function()
{
    msg = "Внимание! Hе найден поток в видео файле.\n" + 
        "Попробуйте выбрать другой эпизод или повторите попытку.";
    AlertPage.show(msg);
};

PlayerPage.onNetworkDisconnected = function()
{
    msg = "Внимание! Нет связи с сервером.\n" + 
        "Проверьте ваше интернет-соединение.";
    AlertPage.show(msg);
};

PlayerPage.onRenderError = function()
{
    msg = "Внимание! Произошла ошибка рендеринга.\n" + 
        "Попробуйте выбрать другой эпизод или повторите попытку.";
    AlertPage.show(msg);
};

PlayerPage.onConnectionFailed = function()
{
    msg = "Внимание! Не удалось присоединиться к серверу.\n" + 
        "Попробуйте выбрать другой эпизод или повторите попытку.";
    AlertPage.show(msg);
};

PlayerPage.deinit = function()
{
    var mwPlugin = document.getElementById("pluginTVMW");
    
    if (mwPlugin && (this.originalSource != null) )
    {
        /* Restore original TV source before closing the widget */
        mwPlugin.SetSource(this.originalSource);
    }
    PlayerPage.plugin.Stop();
    clearInterval(PlayerPage.watchIntervalId);

};
                
PlayerPage.setFullscreen = function()
{
    switch(PlayerPage.current_aspect)
    {
        case PlayerPage.aspects.ASPECT_4_3:
        {
            PlayerPage.plugin.SetDisplayArea(120, 0, 720, 540);
            break;
        }
        case PlayerPage.aspects.ASPECT_16_9:
        default:
        {
            PlayerPage.plugin.SetDisplayArea(0, 0, 960, 540);
            break;
        }
    }
};
        
PlayerPage.request = function(contentType, contentId, videoId, skip_history)
{
    PlayerPage.contentType = contentType;
    PlayerPage.contentId = contentId;
    PlayerPage.videoId = videoId;
    PlayerPage.skip_history = skip_history;
    Main.request([PlayerPage.url,contentType,contentId,videoId].join('/'), PlayerPage.activate);
};
        
PlayerPage.activate = function(data)
{
    Main.setCurrentPage(PlayerPage, PlayerPage.skip_history);
    PlayerPage.skip_history = false;
    PlayerPage.response = data;
    PlayerPage.init();
    PlayerPage.setPrevNext();
    PlayerPage.token = data.token;
    PlayerPage.play();
    setTimeout(function (){
            document.getElementById(PlayerPage.id).style.opacity = 1.0;
            }, 2000);
};
        
PlayerPage.play = function()
{
    if (PlayerPage.response == null)
    {
        Main.debug("No videos to play");
        PlayerPage.onStreamNotFound();
        // well it isn't the same, but I'm lazy to write the text here;
    } else {
        Main.debug("Starting to play url:" + PlayerPage.response.stream);
        PlayerPage.state = PlayerPage.states.PLAYING;
        PlayerPage.plugin.Play(PlayerPage.response.stream);
        var statObj = {
            event_type: "start_playing",
            video_id: PlayerPage.videoId,
            content_type: PlayerPage.contentType,
            content_id: PlayerPage.contentId,
            video_token: PlayerPage.token,
        };
        Stat.notify(statObj, PlayerPage.response.streamLog);
        PlayerPage.watchIntervalId = setInterval(PlayerPage.onWatch, PlayerPage.watchTimeout); 
        PlayerPage.updateVolume();
        PlayerPage.updateState();
    }
};

PlayerPage.setVisible = function(visible)
{
    var v = visible ? "block" : "none";
    document.getElementById(PlayerPage.id).style.display = v;
    if (visible) {
        Utils.addClass(Utils.getById("main"), "transparent");
        Utils.addClass(Utils.getById("header"), "transparent");
        PlayerPage.setSource();
        PlayerPage.updateVolume();
        PlayerPage.updateState();
    } else {
        PlayerPage.plugin.Stop();
        PlayerPage.sendEOPEvent();
        PlayerPage.restoreSource();
        Utils.removeClass(Utils.getById("main"), "transparent");
        Utils.removeClass(Utils.getById("header"), "transparent");
        clearInterval(PlayerPage.watchIntervalId);
    }

    pluginAPI.setOffScreenSaver();
            
};

PlayerPage.setSource = function()
{
    PlayerPage.originalSource = PlayerPage.tvmw.GetSource();
                                                    
        /* Set TV source to media player plugin */
//    PlayerPage.tvmw.SetSource(PlayerPage.tvmw.PL_TVWM_SOURCE_MEDIA);
        PlayerPage.tvmw.SetMediaSource();

};

PlayerPage.restoreSource = function()
{
    if (PlayerPage.originalSource != null) {
        PlayerPage.tvmw.SetSource(PlayerPage.originalSource);
    }
};

PlayerPage.setCurrentTime = function(time)
{
    if (time == PlayerPage.curTime)
        return;
    PlayerPage.curTime = time;
    var curTime = Utils.parseTime(time);
    Utils.putHtml(Utils.getById("pp_info_cur_time"), curTime);
    Utils.putHtml(Utils.getById("pp_status_cur_time"), curTime);

    var el = Utils.getById("pp_progress_bar");
    var percent = parseInt(PlayerPage.curTime*100/PlayerPage.totTime);
    el.style.width = percent+"%";

};

PlayerPage.updateVolume = function()
{
    Main.debug("Volume is " + PlayerPage.no_volume);
    if (PlayerPage.no_volume)
        return;
    var vol = PlayerPage.audioPlugin.GetVolume();
    if (vol<0)
        vol = 0;
    if (vol>100)
        vol = 100;
    Utils.putHtml(Utils.getById("pp_volume_num"), vol);

    var bar = Utils.getById("pp_volume_tribar");
    bar.style.width = vol + "%";

    var mute = PlayerPage.mute;
    Main.debug("Mute is " + mute);
    if (mute) {
        var m = Utils.getById("pp_info_mute").getElementsByTagName("img")[0];
        m.src = "Images/PlayerPage/sound_small_mute.png";
        m = Utils.getById("pp_volume_mute").getElementsByTagName("img")[0];
        m.src = "Images/PlayerPage/sound_big_mute.png";
    } else {
        var m = Utils.getById("pp_info_mute").getElementsByTagName("img")[0];
        m.src = "Images/PlayerPage/sound_small.png";
        m = Utils.getById("pp_volume_mute").getElementsByTagName("img")[0];
        m.src = "Images/PlayerPage/sound_big.png";
    }
};

PlayerPage.updateState = function()
{
    switch(PlayerPage.state)
    {
        case PlayerPage.states.PLAYING:
        {
            var play = Utils.getById("pp_info_state").getElementsByTagName("img")[0];
            play.src = "Images/PlayerPage/play_small.png";
            play = Utils.getById("pp_status_state").getElementsByTagName("img")[0];
            play.src = "Images/PlayerPage/play_big.png";
            break;
        }
        case PlayerPage.states.STOPPED:
        {
            var play = Utils.getById("pp_info_state").getElementsByTagName("img")[0];
            play.src = "Images/PlayerPage/stop_small.png";
            play = Utils.getById("pp_status_state").getElementsByTagName("img")[0];
            play.src = "Images/PlayerPage/stop_big.png";
            break;
        }
        case PlayerPage.states.PAUSED:
        {
            var play = Utils.getById("pp_info_state").getElementsByTagName("img")[0];
            play.src = "Images/PlayerPage/pause_small.png";
            //play = Utils.getById("pp_status_state").getElementsByTagName("img")[0];
            //play.src = "Images/PlayerPage/stop_big.png";
            break;
        }
        default:
        {
            break;
        }
    }
};

PlayerPage.displayVolume = function()
{
    if (PlayerPage.no_volume)
        return;
    var statusbar = Utils.getById("pp_status_bar");
    if (Utils.hasClass(statusbar, "pp_full_bar")) {
        return;
    } else if (Utils.hasClass(statusbar, "pp_volume_bar")) {
        if (PlayerPage.volInterval) 
            return;
            //clearInterval(PlayerPage.volInterval);
    } 

    Utils.removeClass(statusbar, "pp_no_bar");
    Utils.addClass(statusbar, "pp_volume_bar");

    Utils.getById("pp_volume").style.display = 'block';
    PlayerPage.volInterval = setTimeout(function() {
            PlayerPage.volInterval = null;

            var statusbar = Utils.getById("pp_status_bar");
            Utils.addClass(statusbar, "pp_no_bar");
            Utils.removeClass(statusbar, "pp_full_bar");
            Utils.removeClass(statusbar, "pp_volume_bar");

            }, 5000);

};

PlayerPage.displayState = function()
{
    var statusbar = Utils.getById("pp_status_bar");
    if (Utils.hasClass(statusbar, "pp_volume_bar") || (Utils.hasClass(statusbar, "pp_no_bar"))) {
        Utils.removeClass(statusbar, "pp_volume_bar");
        Utils.removeClass(statusbar, "pp_no_bar");
        Utils.addClass(statusbar, "pp_full_bar");
    }

    if (PlayerPage.statInterval) {
        return;
    }
        //clearInterval(PlayerPage.statInterval);
    PlayerPage.statInterval = setTimeout(function() {
            PlayerPage.statInterval = null;

            var statusbar = Utils.getById("pp_status_bar");
            Utils.addClass(statusbar, "pp_no_bar");
            Utils.removeClass(statusbar, "pp_volume_bar");
            Utils.removeClass(statusbar, "pp_full_bar");

            }, 5000);
};


PlayerPage.onInfo = function()
{
    if (PlayerPage.state != PlayerPage.states.PAUSED) {
        var el = document.getElementById("pp_info");
        PlayerPage.updateVolume();
        PlayerPage.updateState();
        el.style.display = el.style.display == 'block' ? "none" : "block"; 
    }
};

PlayerPage.onRight = function()
{
    if (PlayerPage.state != PlayerPage.states.STOPPED) {
        //PlayerPage.onFF();
        return;
    } else {
        if (PlayerPage.response.nextVideo) {
            PlayerPage.request(PlayerPage.contentType, PlayerPage.contentId,
                           PlayerPage.response.nextVideo.videoId, true);
        }
    }
};

PlayerPage.onFF = function()
{
    if (PlayerPage.state == PlayerPage.states.STOPPED) {
        return;
    }
    PlayerPage.plugin.JumpForward(10);
    PlayerPage.updateState();
    if (PlayerPage.state == PlayerPage.states.PLAYING) {
        PlayerPage.displayState();
    }
};

PlayerPage.onLeft = function()
{
    if (PlayerPage.state != PlayerPage.states.STOPPED) {
        //PlayerPage.onRW();
        return;
    } else {
        if (PlayerPage.response.prevVideo) {
            PlayerPage.request(PlayerPage.contentType, PlayerPage.contentId,
                           PlayerPage.response.prevVideo.videoId, true);
        }
    }
};

PlayerPage.onRW = function()
{
    if (PlayerPage.state == PlayerPage.states.STOPPED) {
        return;
    }
    PlayerPage.plugin.JumpBackward(10);
    PlayerPage.updateState();
    if (PlayerPage.state == PlayerPage.states.PLAYING) {
        PlayerPage.displayState();
    }
};

PlayerPage.onPause = function()
{
    switch (PlayerPage.state) 
    {
        case PlayerPage.states.PAUSED:
        case PlayerPage.states.STOPPED:
        {
            /*
            Main.debug("was paused");
            var info = document.getElementById("pp_info");
            info.style.display = PlayerPage.prev_state;
            PlayerPage.hidePrevNext();
            PlayerPage.state = PlayerPage.states.PLAYING;
            PlayerPage.plugin.Resume();
            */
            break;
        }
        case PlayerPage.states.PLAYING:
        {
            Main.debug("was playing");
            var info = document.getElementById("pp_info");
            if (!info.style.display) {
                PlayerPage.prev_state = "none"
            } else {
                PlayerPage.prev_state = info.style.display;
            }
            PlayerPage.updateVolume();
            info.style.display = 'block';
            PlayerPage.state = PlayerPage.states.PAUSED;
            PlayerPage.plugin.Pause();
            break;
        }
        default:
        {
            break;
        }
    }
    PlayerPage.updateState();
    PlayerPage.updateVolume();
};

PlayerPage.onPlay = function()
{
    if (PlayerPage.state == PlayerPage.states.STOPPED) {
        PlayerPage.request(PlayerPage.contentType, PlayerPage.contentId,
            PlayerPage.videoId, true);
        return;
    }
    if (PlayerPage.state == PlayerPage.states.PAUSED) {
            var info = document.getElementById("pp_info");
            info.style.display = PlayerPage.prev_state;
    }
    PlayerPage.state = PlayerPage.states.PLAYING;
    PlayerPage.plugin.Resume();
    if (PlayerPage.mute) {
        PlayerPage.audioPlugin.SetSystemMute(true);
        PlayerPage.audioPlugin.SetUserMute(true);
    }
    PlayerPage.updateState();
    PlayerPage.hidePrevNext();
    //PlayerPage.displayState();
};

PlayerPage.onStop = function(finished)
{
    PlayerPage.plugin.Pause();
    PlayerPage.state = PlayerPage.states.STOPPED;
    PlayerPage.updateState();
    PlayerPage.showPrevNext();
    //PlayerPage.displayState();
    PlayerPage.sendEOPEvent(finished);
    clearInterval(PlayerPage.watchIntervalId);
};

PlayerPage.onMute = function()
{
    if (PlayerPage.no_volume)
        return;
    if (PlayerPage.mute) {
        PlayerPage.audioPlugin.SetSystemMute(false);
        PlayerPage.audioPlugin.SetUserMute(false);
        PlayerPage.mute = false;
    } else { 
        PlayerPage.audioPlugin.SetSystemMute(true);
        PlayerPage.audioPlugin.SetUserMute(true);
        PlayerPage.mute = true;
    }
    PlayerPage.updateVolume();
};

PlayerPage.onPanVolUp = function()
{
    PlayerPage.onVolUp();
};

PlayerPage.onVolUp = function()
{
    if (PlayerPage.no_volume)
        return;
    if (PlayerPage.mute) {
        PlayerPage.audioPlugin.SetSystemMute(false);
        PlayerPage.audioPlugin.SetUserMute(false);
        PlayerPage.mute = false;
    }
    PlayerPage.audioPlugin.SetVolumeWithKey(0); // should inc vol
    PlayerPage.updateVolume();
    PlayerPage.displayVolume();
};

PlayerPage.onPanVolDown = function()
{
    PlayerPage.onVolDown();
};

PlayerPage.onVolDown = function()
{
    if (PlayerPage.no_volume)
        return;
    if (PlayerPage.mute) {
        PlayerPage.audioPlugin.SetSystemMute(false);
        PlayerPage.audioPlugin.SetUserMute(false);
        PlayerPage.mute = false;
    }
    PlayerPage.audioPlugin.SetVolumeWithKey(1); // should dec vol
    PlayerPage.updateVolume();
    PlayerPage.displayVolume();
};

PlayerPage.showPrevNext = function()
{
    if (PlayerPage.response.nextVideo) {
        var el = Utils.getById("pp_item_next");
        el.style.display = "block";
    }
    if (PlayerPage.response.prevVideo) {
        var el = Utils.getById("pp_item_prev");
        el.style.display = "block";
    }
    var statusbar = Utils.getById("pp_status_bar");
    Utils.removeClass(statusbar, "pp_no_bar");
    Utils.removeClass(statusbar, "pp_volume_bar");
    Utils.addClass(statusbar, "pp_full_bar");
};

PlayerPage.hidePrevNext = function()
{
    var el = Utils.getById("pp_item_next");
    el.style.display = "none";
    el = Utils.getById("pp_item_prev");
    el.style.display = "none";
    var statusbar = Utils.getById("pp_status_bar");
    Utils.removeClass(statusbar, "pp_full_bar");
    Utils.removeClass(statusbar, "pp_volume_bar");
    Utils.addClass(statusbar, "pp_no_bar");
};

PlayerPage.setPrevNext = function()
{
    if (!(PlayerPage.response.prevVideo)) { 
    } else { 
        var pp_item = Utils.getById("pp_item_prev");
        var img = pp_item.getElementsByTagName("img")[0];
        Utils.setImage(img, PlayerPage.response.prevVideo.image);
        var span = pp_item.getElementsByTagName("span")[0];
        Utils.putHtml(span, PlayerPage.response.prevVideo.title);
    }
    if (!(PlayerPage.response.nextVideo)) { 
    } else { 
        var pp_item = Utils.getById("pp_item_next");
        var img = pp_item.getElementsByTagName("img")[0];
        Utils.setImage(img, PlayerPage.response.nextVideo.image);
        var span = pp_item.getElementsByTagName("span")[0];
        Utils.putHtml(span, PlayerPage.response.nextVideo.title);
    }
};

PlayerPage.onRenderingComplete = function()
{
    PlayerPage.onStop(true);
};

PlayerPage.onAspect = function()
{
    if (PlayerPage.current_aspect === PlayerPage.aspects.ASPECT_16_9)
        PlayerPage.current_aspect = PlayerPage.aspects.ASPECT_4_3;
    else 
        PlayerPage.current_aspect = PlayerPage.aspects.ASPECT_16_9;

    PlayerPage.setFullscreen();
};

PlayerPage.onWatch = function()
{
    if (PlayerPage.state !== PlayerPage.states.STOPPED)  {
        var statObj = {
            event_type: "playing",
            event_info: parseInt(PlayerPage.curTime),
            video_id: PlayerPage.videoId,
            content_type: PlayerPage.contentType,
            content_id: PlayerPage.contentId,
            video_token: PlayerPage.token,
        };
        Stat.notify(statObj, PlayerPage.response.streamLog);
    }
};

PlayerPage.notifyStat = function(skipped)
{
    // Dummy, don't send anything
    // Will send all the info when player runs
};

PlayerPage.sendEOPEvent = function(finished)
{
    var curTime;
    if (finished === undefined) {
        curTime = PlayerPage.curTime;
    } else {
        curTime = 'finished';
    }
    var statObj = {
        event_type: "end_of_playing",
        event_info: curTime,
        video_id: PlayerPage.videoId,
        content_type: PlayerPage.contentType,
        content_id: PlayerPage.contentId,
        video_token: PlayerPage.token,
    };
    Stat.notify(statObj, PlayerPage.response.streamLog);

};
