var Utils = {};

/* Returns element by elementId */
Utils.getById = function(elementId) {
    return document.getElementById(elementId);
}

/* Returns first child element of parent with `tag` name. parent could
 * be dom element or string with element id. */
Utils.getByTag = function(parent, tag) {
    var element = null;

    if (typeof parent == "string") {
        parent = document.getElementById(parent);
    }

    if (parent !== null) {
        var elements = parent.getElementsByTagName(tag);
        if (elements.length)
        {
            element = elements[0];
        }
    }

    return element;
}

/* Returns true if element `ele` has class `cls`.*/
Utils.hasClass = function(ele, cls) {
    if (ele) {
        return ele.className.match(new RegExp('(\\s|^)'+cls+'(\\s|$)'));
    }
    return false;
};

/* Adds class `cls` to element `ele` */
Utils.addClass = function(ele, cls) {
    if (ele && !Utils.hasClass(ele,cls)) {
        ele.className += " "+cls;
    }
};

/* Removes class `cls` from element `ele` */
Utils.removeClass = function(ele, cls) {
    if (ele && Utils.hasClass(ele,cls)) {
	var reg = new RegExp('(\\s|^)'+cls+'(\\s|$)');
	ele.className=ele.className.replace(reg,' ');
    }
};

/* Sets inner html of dom element using widgetAPI.putInnerHTML. */
Utils.putHtml = function(element, html) {
    if (element) {
        return widgetAPI.putInnerHTML(element, html);
    }
};

/* Sets source of image `element` according to image url
 * template. Image element must have width and height attributes with
 * desired values. */
Utils.setImage = function(element, imageUrlTemplate) {
    if (element && element.width && element.height) {
        var url = imageUrlTemplate.replace("{action}", "crop");
        url = url.replace("{width}", element.width);
        url = url.replace("{height}", element.height);
        element.src = url;
    }
}

/* get time in msecs or secs and parse it to 'h:mm:ss' fmt */
Utils.parseTime = function(t, secs)
{
    var time = parseInt(t);
    if (typeof secs === "undefined") {
        time = parseInt(time/1000);  
    }
    var hours = parseInt(time/3600);
    var min = parseInt(time/60) % 60;
    var sec = time % 60; 

    var strTime = hours + ':' + (min<10 ? '0' + min : min) + ':' + (sec<10 ? '0' + sec : sec);

    return strTime;

};

Utils.capitalize = function(str)
{
    if (str) {
        return str.charAt(0).toUpperCase() + str.slice(1);
    }

    return "";
};
